#!/usr/bin/env python3

# Convert pdf to text using pdftotext first

import sys
import regex
import json


def get_args():
    """ Get command line arguments """

    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("txt_in")
    parser.add_argument("json_out")
    parser.add_argument("txt_out")
    return parser.parse_args()


def main(args):
    """ Entry point for script """
    items = []
    codes = set()
    code_pattern = regex.compile(r'[A-Z]{2}-[A-Z]\d{5}')
    price_pattern = regex.compile(r'£?\d+\.\d\d')
    unit_pattern = regex.compile(r'(EA|SQM|SHT|M|PCK|BOX|RL|LEN|MTR|PK|REEL|L|)')
    category_pattern = regex.compile(r'[A-Z]{2}-[A-Z]\d\d')

    cur = []

    with open(args.txt_in) as fp:
        for line in fp:
            line = line.strip()
            #print(f"{line=} {len(cur)=}")
            empty = line == ''
            if empty:
                continue

            is_code = code_pattern.fullmatch(line) is not None
            if is_code:
                codes.add(line)

            if len(cur) == 0:
                if is_code:
                    cur.append(line)
                continue

            is_product_name = True
            if len(cur) == 1:
                if is_code:
                    cur = [line]
                    continue

                lowercase_count = 0
                uppercase_count = 0
                for char in line:
                    if not char.isalpha():
                        continue

                    if char.islower():
                        lowercase_count += 1
                    else:
                        uppercase_count += 1

                # all product names are uppercase so skip anything that isn't mostly uppercase
                is_product_name = len(line) > 5 and uppercase_count > lowercase_count and category_pattern.fullmatch(line) is None

                if is_product_name:
                    cur.append(line)
                continue

            is_unit = unit_pattern.fullmatch(line) is not None
            if len(cur) == 2:
                if is_unit:
                    cur.append(line)
                else:
                    if is_code:
                        cur = [line]
                        continue
                    print(f"Unkown unit? {line=}")
                continue

            is_price = price_pattern.fullmatch(line) is not None
            if len(cur) == 3:
                if is_price:
                    cur.append(line)
                    items.append(cur)
                    #print(f"{cur=}")
                    cur = []
                continue


    item_codes = [i[0] for i in items]
    print(f"{len(codes)=} {len(items)=}")


    missing = [ c for c in codes if c not in item_codes ]

    with open(args.json_out, "w") as fp:
        fp.write(json.dumps(items))

    with open(args.txt_out, "w") as fp:
        fp.write("\n".join(["\t".join(item) for item in items]))


    #print(json.dumps(items))

    return 0


if __name__ == '__main__':
    try:
        sys.exit(main(get_args()))
    except KeyboardInterrupt:
        sys.exit(0)
